function [Tsel,Close,Fit,CloseFast,V,flag,S,up,down] = SPSindsampBayesLaplace(G,y,fs,q,m,f,flag,o2,N1)%,Close,CloseFast,V

%INPUT
%G=regression matrix
%y=output data
%fs=the estimate of f
%q=confidence
%m=number of signed
%flag = 0 or 1 (ReLS2 is activated when flag=1)
%OUTPUT
%Tsel=non uniform samples

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%USEFUL VARIABLES p,n,Rn,iR%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
p=size(G,2);%length of theta
n=length(y);%length of y
Rn=G'*G/n;
sqrRn=Rn^0.5;
iRn=inv(Rn);%Rn^-1;
%utest=randn(1,2000);ztest=filter(f,1,utest);
Tsel=[];
Runs=5000;%# of samples from each ellipsoid
nMCMC=2*Runs;%#of MCMC samples
con=0;%to list the signs from 1 to (m-1)*n
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%S,R,Ter (selecting the case by flag) in Ter%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Cases=string({'LS+SPS','ReLS+SPS1','ReLS+SPS2','ReLS+SPS1g','ReLS+SPS2g','ReLS+SPS3','ReLS+SPS4','LASSO','VarSel'});
S=sign(randn(1,(m-1)*n));
R2=sqrt(o2)*sign(randn(1,(m-1)*n)).*exprnd(1,1,(m-1)*n)/sqrt(2);%for measurement noise
R3=sqrt(o2)*randn(1,(m-1)*n);%for theta
R4=sqrt(o2)*randn(1,(m-1)*n);%for first N1 measurements
%R2=sqrt(o2)*sign(randn(p,1)).*exprnd(1,1,(m-1)*n)/sqrt(2);
%disp('Forse segno opposto per R3(i)?')
Ter(1:(n-p))=2;%Laplacian noise with R2
Ter(n-p+1:n)=3;%Gaussian pdf on f with R3
Ter(1:N1)=4;%Gaussian noise with R4 for first 20 measurements
flag
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n
size(Ter)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%THE ELLIPSOIDS%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:(m-1);
    %%%%%A{i},B{i},C{i} to define (x-B{i})'*A{i}*(x-B{i})<=C{i}%%%
    Q{i}=zeros(p,p);
    psi{i}=zeros(p,1);
    for t=1:n;
        con=con+1;
        Q{i}=Q{i}+(Ter(t)==0)*G(t,:)'*G(t,:)/n+(Ter(t)==1)*S(con)*G(t,:)'*G(t,:)/n;
        psi{i}=psi{i}+(Ter(t)==0)*G(t,:)'*y(t)/n+(Ter(t)==1)*S(con)*G(t,:)'*y(t)/n+(Ter(t)==2)*R2(con)*G(t,:)'/n+...
            (Ter(t)==3)*R3(con)*G(t,:)'/n+(Ter(t)==4)*R4(con)*G(t,:)'/n;
    end
    A{i}=Rn-Q{i}*iRn*Q{i};iA{i}=inv(A{i});
    b{i}=Q{i}*iRn*psi{i}-Rn*fs;
    B{i}=-iA{i}*b{i};
    c{i}=fs'*Rn*fs-psi{i}'*iRn*psi{i};
    C{i}=-c{i}+b{i}'*iA{i}*b{i};
    %%%%SAMPLES and VOLUMES%%%
    [T,~,~,~,up(i,:),down(i,:)]=Gaussgibbs(iA{i},C{i},B{i},B{i},Runs);
    Tsel=[Tsel;T];%samples in original coordinates
    logV(i)=log(((pi^(p/2))/(gamma(p/2+1))))+0.5*sum(log(eig(iA{i}*C{i})));
    V=exp(logV);
end
toc
up=max(up);
down=min(down);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%MCMC FOR UNIFORM SAMPLES AND CLOSE%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
try
    Prob=V/sum(V);
    Xsel=Tsel(1,:)';
    selX=1;
    for i=1:(m-1)
        BelongX(i)=((Tsel(selX,:)'-B{i})'*A{i}*(Tsel(selX,:)'-B{i})<=C{i});%ellipsoids flag
    end
    for k=1:nMCMC
        q=randsample(m-1,1,true,Prob);%randomly chosen ellipsoid
        selY=(q-1)*Runs+ceil(Runs*rand);
        Ysel=Tsel(selY,:)';
        for i=1:(m-1)
            BelongY(i)=((Tsel(selY,:)'-B{i})'*A{i}*(Tsel(selY,:)'-B{i})<=C{i});%ellipsoids flag
        end
        U=rand;
        if U<(sum(BelongX)/sum(BelongY))
            accX(k,:)=Ysel';
            Xsel=Ysel;
            BelongX=BelongY;
        else
            accX(k,:)=Xsel';
        end
    end
    centroid=mean(accX);
    for k=1:nMCMC;
        q3(k)=norm(centroid-accX(k,:));
        q4(k)=100-100*norm(f'-accX(k,:))/norm(f);
    end
    Close=mean(q3);clear q3
    Fit=mean(q4);clear q4
    %Z1=min(Ztest);Z2=max(Ztest);sum(Z1<ztest & ztest<Z2)/2000
catch
    Close=+inf;
    Fit=+inf;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%flag on f%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for i=1:(m-1)
    Belongf(i)=((f-B{i})'*A{i}*(f-B{i})<=C{i});%ellipsoids flag
end
flag=sum(Belongf)>0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%CLOSEFAST%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
try
    [q1,q2]=max(V);
    centroid=mean(Tsel((q2-1)*Runs+1:q2*Runs,:));
    for k=1:Runs;
        q3(k)=norm(centroid-Tsel((q2-1)*Runs+k,:));
    end
    CloseFast=mean(q3);
catch
    CloseFast=+inf;
end
toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


end