
function [Tsel,Close,FitCI,flag,up,down]=Gaussgibbs(E,k,f,fs,Runs)
%INPUT
%E = posterior covariance
%f = true
%k and fs s.t. (x-fs)'inv(E)*(x-fs)<=k, f=true
%Runs=# of samples to generate
%OUTPUT
%Tsel=chosen theta
%FitCI=distance in fit from f and Tsel, i.e. fit measure of the CI
%Closeness{i} CI dispersion from the centroid (i=1) and fs (i=2)
%flag(i) = 1 if f is contained

E=0.5*(E+E');

try
nz=length(E);
X_Cnz=randn(nz,Runs);
X_Cnz=X_Cnz./kron(ones(nz,1),sqrt(sum(X_Cnz.^2))); % Points uniformly distributed on hypersphere
R=ones(nz,1)*(rand(1,Runs).^(1/nz)); 
unif_sph=R.*X_Cnz; % Runs points in the hypersphere
T=chol(E); % Cholesky factorization of E => E=T’T
unif_ell =T'*unif_sph ; % Hypersphere to hyperellipsoid mapping
Tsel=(unif_ell*sqrt(k)+(fs*ones(1,Runs)))'; % Translation around gate center

centroid=mean(Tsel);
iE=inv(E);

for gian=1:size(Tsel,1)
    FitCI(gian)=100-100*(norm(f-Tsel(gian,:)')/norm(f));
    Closeness{1}(gian)=norm(centroid-Tsel(gian,:));
    %Closeness{2}(gian)=norm(fs-Tsel(gian,:)');
    %if ((Tsel(gian,:)'-fs)'*iE*(Tsel(gian,:)'-fs)>k);disp('Problem');pause;else;disp('ok');end
end
Close=mean(Closeness{1});

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% FLAG ON f %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
flag=((f-fs)'*iE*(f-fs)<=k);
Delta=sqrt(k*diag(E));
up=fs+Delta;
down=fs-Delta;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%% up,down,FitCI,Closeness %%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%up=max(Tsel);
%down=min(Tsel);
FitCI=mean(FitCI);%max(FitCI)*(flag(1)==0)+100*(flag(1)==1);
%Closeness=mean(Closeness);

catch
    Tsel=fs';
    FitCI=[];
    Closeness=[];
    flag=[];
end

